/*

  zeit - some time management routines
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __YYYZEIT_HPP
#define __YYYZEIT_HPP

/*
  the zeit class has the meaning of a relative time interval.
  if someone uses a zeit in a absolute way, it means a relative time interval
  relative to 1970-01-01 00:00:00 UTC.
*/

#include "binarydefs.hpp"

#include <sys/time.h>

namespace yyyzeit {

class zeit {
public:
  typedef int64 reptype;

  zeit () {}; // undefined value !!!

  inline void setmicroseconds(const reptype x) { microsec=x; };
  inline const reptype getmicroseconds() const { return microsec; };

  void setseconds_asdouble (const double seconds); // NOT microseconds !!!
  const double getseconds_asdouble () const;       // NOT microseconds !!!

  inline const bool isnegative() const { return microsec<0; }; // 0 is not negative
  void convert_to_timeval (timeval &tv) const; // this can silently overflow if the time doesn't fit in

  inline const bool operator < (const zeit &x) const { return microsec < x.microsec; };
  inline const bool operator == (const zeit &x) const { return microsec == x.microsec; };
  inline const bool operator != (const zeit &x) const { return microsec != x.microsec; };
  inline const bool operator > (const zeit &x) const { return microsec > x.microsec; };
  inline const bool operator <= (const zeit &x) const { return microsec <= x.microsec; };
  inline const bool operator >= (const zeit &x) const { return microsec >= x.microsec; };

  inline void operator += (const zeit &x) { microsec += x.microsec; };
  inline void operator -= (const zeit &x) { microsec -= x.microsec; };

private:
  reptype microsec;
};

inline zeit operator - (const zeit &a, const zeit &b)
{
  zeit t;
  t.setmicroseconds (a.getmicroseconds() - b.getmicroseconds());
  return t;
};

inline zeit operator + (const zeit &a, const zeit &b)
{
  zeit t;
  t.setmicroseconds (a.getmicroseconds() + b.getmicroseconds());
  return t;
};


// construct a zeit object
inline zeit seconds (const zeit::reptype x)
{
  zeit t;
  t.setmicroseconds(x*1000000);
  return t;
};

// construct a zeit object
inline zeit microseconds (const zeit::reptype x)
{
  zeit t;
  t.setmicroseconds(x);
  return t;
};

// construct a zeit object
inline zeit zerotime ()
{
  zeit t;
  t.setmicroseconds(0);
  return t;
};

// return current absolute system time
const zeit getcurrenttime ();

}

#endif
